#include "paddle.hpp"

#include "general.hpp"

#include <cmath>


using namespace frog::geo;


std::optional<vec2> frog::geo::collision_2D(geo::paddle paddle,
                                            polar2 coords,
                                            float radius)
{
    if (coords.r + radius < paddle.pos.r)
        return std::nullopt;

    if (coords.r - radius > paddle.pos.r + paddle.thickness)
        return std::nullopt;

    float diff = angle_diff(coords.theta,
                           paddle.pos.theta + paddle.angular_size / 2);

    float angular_radius = std::atan2(radius, coords.r);

    if (std::abs(diff) > paddle.angular_size / 2 + angular_radius)
        return std::nullopt;

    if (std::abs(diff) > paddle.angular_size / 2)
    {
        vec3 up = { 0, 1, 0 };

        vec2 pos2 = geo::to_cartesian(coords);
        vec3 vec = { pos2.x(), 0, pos2.y() };

        vec3 normal = vec.cross(up) * (diff > 0 ? -1 : 1);

        return vec2{ normal.x(), normal.z() };
    }

    bool inside = coords.r < paddle.pos.r + paddle.thickness / 2;

    vec2 normal = geo::to_cartesian(coords) * (inside ? -1 : 1);

    return { normal.normalized() };
}
