#pragma once

#include "core/script.hpp"

#include "constants.hpp"


struct ball_script : frog::script
{
    frog::geo::vec3 vector;

    void init(frog::game_object& obj, frog::engine&) override
    {
        using namespace frog;

        obj.model().pos = geo::vec3{ 0, Ball::Radius, 0 };

        vector = { 1, 0, 1 };
    }

    void stable_update(frog::game_object& obj, frog::engine& engine) override
    {
        using namespace frog;

        if (engine.input->is_down(os::key{ GLFW_KEY_SPACE }))
        {
            obj.model().pos = geo::vec3{ 0, Ball::Radius, 4 };
        }

        obj.model().pos += vector * engine.global->stable_frame_time();

        float radius = Ball::Radius;

        auto polar_pos = geo::to_polar({ obj.model().pos.x(),
                                         obj.model().pos.z() });

        auto bounce = [&](auto& wall)
        {
            auto found = wall.get_script<bounce_script>();

            if (!found)
                return;

            auto paddle = found->paddle;

            auto normal = geo::collision_2D(paddle, polar_pos, radius);

            if (normal)
            {
                float len = vector.length();
                vector = geo::reflect(vector, geo::to_3D(*normal));
                vector /= len;
            }
        };

        engine.scenes->current().for_each_object(frog::by_tag("wall"),
            [&](auto& wall)
            {
                auto wall_paddle = wall.get_script<wall_script>()->paddle;

                auto normal = geo::collision_2D(wall_paddle, polar_pos, radius);

                if (normal)
                {
                    vector = geo::reflect(vector, geo::to_3D(*normal));
                }
            });

        engine.scenes->current().for_each_object(frog::by_tag("paddle"),
            [&](auto& wall)
        {
            auto paddle = wall.get_script<paddle_script>()->paddle;

            auto normal = geo::collision_2D(paddle, polar_pos, radius);

            if (normal)
            {
                vector = geo::reflect(vector, geo::to_3D(*normal));
            }
        });
    }

    void frame_update(frog::game_object&, frog::engine&) override {}

    void destroyed(frog::game_object&, frog::engine&) override {}
};
