#pragma once

#include "core/script.hpp"
#include "geometry/general.hpp"
#include "geometry/paddle.hpp"

#include "constants.hpp"


struct paddle_script : frog::script
{
    frog::geo::paddle paddle;

    paddle_script(float starting_angle)
        : paddle(create_paddle(starting_angle)) {}

    void init(frog::game_object& object, frog::engine&) override
    {
        using namespace frog;

        object.model().rot.y() = paddle.pos.theta * geo::ToRad;
    }

    void stable_update(frog::game_object& object, frog::engine& engine) override
    {
        using namespace frog;

        double time = engine.global->stable_frame_time();

        float move = 0;

        if (engine.input->is_down(os::key{ GLFW_KEY_D }))
        {
            move = -1;
        }
        if (engine.input->is_down(os::key{ GLFW_KEY_A }))
        {
            move = 1;
        }

        paddle.pos.theta += move * Paddle::Speed * geo::ToRad * time;

        object.model().rot.y() = paddle.pos.theta;
    }

    static frog::geo::paddle create_paddle(float start)
    {
        auto result = frog::geo::paddle{};
        result.angular_size = frog::geo::ToRad * Paddle::AngularSize;
        result.height = Paddle::Height;
        result.thickness = Paddle::Thickness;
        result.pos.r = Paddle::Radius;
        result.pos.theta = frog::geo::ToRad * start;
        result.pos.y = 0;

        return result;
    }
};
